# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use strict;
use defsjukebox; # Get the settings entered by the user during configuration
use DBI;
use Getopt::Long;

Getopt::Long::Configure("pass_through","no_ignore_case");

my $help=0;
my $minLen=0;
my $skip=0;
my %songid;

GetOptions("help|h",\$help,
	   "minlength|m=i",\$minLen,
	   "skip|s",\$skip);

if ($help||int(@ARGV)>0) {
    print <<__EOT__;
Usage:

    $0 {options}

Options:
  --h|help                Display this message
  --m|minlength={number}  Minimum number of songs found to create album
  --s|skip                Allow skips in tracknumber sequence

__EOT__
    exit 2;
}

if ($minLen<=0) {
    $minLen=2;
}

my $dbh=DBI->connect("dbi:mysql:database=$DBNAME",$DBUSER,$DBPASS) || die;

# Initialise songs

{
    my $songs=$dbh->prepare("select id from songs") || die;
    $songs->execute();
    while(my($id)=$songs->fetchrow_array()) {
	$songid{$id}=1;
    }
    
    my $progsongs=$dbh->prepare("select distinct songid from programmember") || die;
    $progsongs->execute || die;
    while(my($id)=$progsongs->fetchrow_array()) {
	delete $songid{$id};
    }
}

# Actually create program

my $info=$dbh->prepare("select artist,title,volume,trackno from songs where id = ?") || die;
my $programadd=$dbh->prepare("insert into programs values (0,?,NULL,?,NULL,NULL)") || die;
my $songadd=$dbh->prepare("insert into programmember values (0,?,?)") || die;

sub createProgram {
    my $artist=shift(@_);
    my $volume=shift(@_);
    print "Creating $artist - $volume consisting of:\n";
    $programadd->execute($volume,$artist) || die;   # JBM - switched around to match SQL
    my $insertId = $programadd->{'mysql_insertid'};
    for my $id (@_) {
	$songadd->execute($insertId,$id) || die;
	$info->execute($id) || die;
	print("  $insertId,$id: ".join(' - ',$info->fetchrow_array())."\n");
	delete $songid{$id};
    }
}

# Add songs with artist - album matching first.

{
    my $lstArtist="";
    my $lstVolume="";

    my $artists=$dbh->prepare(<<__EOSQL__) || die;
select id,volume,artist,trackno
  from songs
 where volume is not null and artist is not null and trackno is not null
 order by volume,artist,trackno
__EOSQL__

    $artists->execute() || die;

    my $ok=0;
    my @songs;
    
    while(my($id,$volume,$artist,$trackno)=$artists->fetchrow_array()) {
	if ($songid{$id}) {
	    if ($artist ne $lstArtist ||
		$volume ne $lstVolume) {
		if ($ok&&int(@songs)>=$minLen) {
		    &createProgram($lstArtist,$lstVolume,@songs);
		}
		undef @songs;
		$ok=1;
	    }
	    
	    push(@songs,$id);
	    
	    if ($skip||int(@songs)!=$trackno) {
		$ok=0;
	    }
	    
	    $lstArtist=$artist;
	    $lstVolume=$volume;
	}
    }
    if ($ok&&int(@songs)>=$minLen) { # JBM end case for final Album. Otherwise Zooropa doesn't get created
	&createProgram($lstArtist,$lstVolume,@songs);
    }
}


# Add albums with mixing artists last.

{
    my $lstVolume="";

    my $various=$dbh->prepare(<<__EOSQL__) || die;
select id,volume,trackno
  from songs
 where volume is not null and trackno is not null
 order by volume,trackno
__EOSQL__
    $various->execute() || die;

    my $ok=0;
    my @songs;
    
    while(my ($id,$volume,$trackno)=$various->fetchrow_array()) {
	if ($songid{$id}) {
	    if ($volume ne $lstVolume) {
		if ($ok&&int(@songs)>=$minLen) {
		    &createProgram(undef,$lstVolume,@songs);
		}
		undef @songs;
		$ok=1;
	    }
	    
	    push(@songs,$id);
	    
	    if ($skip||int(@songs)!=$trackno) {
		$ok=0;
	    }
	    
	    $lstVolume=$volume;
	}
    }
    if ($ok&&int(@songs)>=$minLen) { #end case
       &createProgram(undef,$lstVolume,@songs);
    }
}

undef $info;
undef $programadd;
undef $songadd;
$dbh->disconnect;
