# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use strict;
use defsjukebox; # Get the settings entered by the user during configuration

use DBI;

my $dbh=DBI->connect("dbi:mysql:database=$DBNAME",$DBUSER,$DBPASS) || die;

my $force=0;
my $del=0;

if ($ARGV[0]&&$ARGV[0] eq "force") {
    $force=1;
}
if ($ARGV[0]&&$ARGV[0] eq "del") {
    $del=1;
}

sub fetchsetting {
    my $res=$dbh->prepare("select value from settings where name = ?");
    $res->execute($_[0]);
    my $ret;
    ($ret)=$res->fetchrow_array();
    return $ret;
}

my $orgfrm=fetchsetting("ORGFORMAT");
my $orgspc="a-zA-Z0-9";
$orgspc=~s/([^-a-zA-Z0-9])/\\$1/g;

my $ren=$dbh->prepare("update songs set artist = ?, title = ?, volume = ? where id = ?");
my $mov=$dbh->prepare("update songs set file = ?,volid = ? where id = ?");
my $renalb=$dbh->prepare("update programs set name = ?, artist = ? where id = ?");

my %validVol;
{
    my $vol=$dbh->prepare("select id,mounted from volumes where (flags&6)=2 and length(mounted)>1");
    $vol->execute();
    while(my ($volid,$mount)=$vol->fetchrow_array()) {
	$validVol{$volid}=$mount;
    }
}

my %skip;

{
    my $ripq=$dbh->prepare("select songid from ripqueue where songid > 0");
    $ripq->execute();
    while(my ($songid)=$ripq->fetchrow_array) {
	$skip{$songid}=1;
    }
}

sub unThe {
    my ($str)=@_;
    $str=~s/^(the)\s+(.*)$/$2, The/i;
    return $str;
}

sub unSpecial {
    my ($str)=@_;
    $str=~s/[^$orgspc]+/_/g;
    return $str;
}

{
    my $alb=$dbh->prepare("select id,artist,name from programs");
    $alb->execute();
    while(my ($id,$artist,$album)=$alb->fetchrow_array) {
	$artist=&unThe($artist);
	$album=&unThe($album);
	$renalb->execute($album,$artist,$id);
    }
}

my $sth=$dbh->prepare(<<__EOSQL__);
SELECT
       a.id,
       a.file,
       a.artist,
       a.title,
       a.volume,
       a.trackno,
       a.user,
       a.format,
       a.volid,
       b.artist,
       b.name
  FROM songs a
  LEFT JOIN programmember c ON c.songid = a.id
  LEFT JOIN programs b ON b.id = c.programid
 WHERE a.format != 'cd'
__EOSQL__

$sth->execute();

my %ids;

while(my ($id,$file,$artist,$title,$album,$trackno,$user,$format,$volid,$aartist,$aalbum)=$sth->fetchrow()) {
    if (!$ids{$id}) {
	if ($validVol{$volid}||!$volid) {
	    if ($validVol{$volid}) {
		$file=$validVol{$volid}."/".$file;
	    }
	    if (!-e $file) {
		if ($del) {
		    print "The file $file is non-existant, removing from the database.\n";
		    my $res=$dbh->prepare("delete from songs where id = ?");
		    $res->execute($id);
		    $res=$dbh->prepare("delete from ripqueue where songid = ?");
		    $res->execute($id);
		    $res=$dbh->prepare("delete from groupmember where songid = ?");
		    $res->execute($id);
		    $res=$dbh->prepare("delete from queue where id = ?");
		    $res->execute($id);
		    $res=$dbh->prepare("delete from programmember where songid = ?");
		    $res->execute($id);
		} else {
		    print "The file $file is non-existant, use \"$0 del\" to remove it from DB.\n";
		}
	    } elsif (!$skip{$id}) {
		$artist=&unThe($artist);
		$title=&unThe($title);
		$album=&unThe($album);
		
		$ren->execute($artist,$title,$album,$id);
		
		my $fnam=$orgfrm;
		
		if (!$trackno) {
		    $trackno="";
		}
		if (!$aalbum) {
		    $aartist="_Unsorted";
		    $aalbum=$artist;
		} elsif (!$aartist) {
		    $aartist="_Various";
		}
		
		$fnam=~s/%([^a-z0-9]*)([a-zA-Z0-9])/do {
		    my $res="";
		    if ($2 eq "a") {
			$res=$artist;
		    } elsif ($2 eq "A") {
			$res=$aartist;
		    } elsif ($2 eq "t") {
			$res=$title;
		    } elsif ($2 eq "b") {
			$res=$album;
		    } elsif ($2 eq "B") {
			$res=$aalbum;
		    } elsif ($2 eq "n") {
			$res=$trackno;
		    } elsif ($2 eq "N") {
			if ($trackno) {
			    $res=sprintf("%02d",$trackno);
			}
		    } elsif ($2 eq "u") {
			$res=$user;
		    } elsif ($2 eq "f") {
			$res=$format;
		    }
		    $res=&unSpecial($res);
		    if (length($res)>0) {
			$res.=$1;
		    }
		    $res;
		}/eig;
		
		my $realfile;
		my $maxSiz=0;
		
		if ($volid) {
		    $realfile=$validVol{$volid}."/".$fnam;
		} else {
		    for my $nvol (keys %validVol) {
			my $base=$validVol{$nvol};
			if (substr($file,0,length($base)) eq $base) {
			    if ($maxSiz<length($base)) {
				$maxSiz=length($base);
				$volid=$nvol;
				print "Moving $file to volume mounted on ".$validVol{$nvol}."\n";
			    }
			}
		    }
		    
		    if ($volid) {
			$realfile=$validVol{$volid}."/".$fnam;
		    } else {
			$realfile=$fnam="$TARGETBASE/$fnam";
		    }
		}
		
		if ($file ne $realfile) {
		    if (-f $realfile&&!$force) {
			print "ERROR: New name $realfile occupied, use \"$0 force\" to overwrite\n";
		    } elsif (-w $file) {
			print "Moving $file to $realfile\n";
			my ($dirname)=($realfile=~/^(.*)\/[^\/]+$/);
			if (!-d $dirname) {
			    system($MKDIR,"-p",$dirname);
			}
			if ((!(system($MV,$file,$realfile)>>8))||$force) {
			    $mov->execute($fnam,$volid,$id);
			} else {
			    print "ERROR: Couldn't move $file to $realfile\n";
			}
		    } elsif ($maxSiz) {
			print "ERROR: File $file not writable, still moving to new volume\n";
			$file=substr($file,$maxSiz);
			$file=~s/^\///;
			$mov->execute($file,$volid,$id);
		    } else {
			print "ERROR: File $file not writable\n";
		    }
		} elsif ($maxSiz) { # Needed if volume change without actual file moved
		    $mov->execute($fnam,$volid,$id);
		}
		$ids{$id}=$realfile;
	    }
	}
    }
}
