# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use strict;
use defsjukebox; # Get the settings entered by the user during configuration

$SIG{CHLD}='IGNORE';

my $orguser=shift(@ARGV);

if (!$orguser) {
    exit 0
	if (fork);
}

my $user="";

system("renice -10 $$");

my $pid;

my @pids;

sub resignal {
    kill $_[0],$pid;
}

sub stopsig {
    kill "STOP",$pid;
}

use DBI;

my $dbh=DBI->connect("dbi:mysql:database=$DBNAME",$DBUSER,$DBPASS) || die;

sub fetchsetting {
    my $res=$dbh->prepare("select value from usersetting where name = ? and user = ?");
    $res->execute($_[0],$orguser);
    my $ret;

    if (!(($ret)=$res->fetchrow_array())) {
	$res=$dbh->prepare("select value from settings where name = ?");
	$res->execute($_[0]);
	($ret)=$res->fetchrow_array();
    }
    return $ret;
}

sub toplist {
    my $toplist=fetchsetting("TOPLIST");
    my $topk=fetchsetting("TOPLISTK")**(1.0/7.0);
    $topk=1.2
	if (!$topk);
    return $topk**((time()-902940490)/86400);
}

my $deltop=fetchsetting("DELTOP");
my $setlen=$dbh->prepare("update settings set value = ? where name = 'SONGLEN'");
my $setpos=$dbh->prepare("update settings set value = ? where name = 'SONGPOS'");
my $updtop=$dbh->prepare("update songs set toplist = toplist - ? where file = ?");

sub startplayer {
    my ($prog,$file,$flags) = @_;
    if (!$orguser) {
	$setlen->execute(0);
	$setpos->execute(0);
    }
    $pid=(open(PLAYER,"-|") || do {
	exec ($prog,$file);
	die "Failed to run $prog\n";
    });
    $SIG{CONT}=\&resignal;
    if (!$flags&&$file!~/:\/\//) {
	$SIG{INT}=\&resignal;
	$SIG{HUP}=\&resignal;
	$SIG{TSTP}=\&resignal;
    } else {
	$SIG{TSTP}=\&stopsig;
	$SIG{INT}='IGNORE';
	$SIG{HUP}='IGNORE';
    }
    $SIG{TERM}=\&resignal;
    my ($cur,$tot);
    while (<PLAYER>) {
	if (/^\s*(\d+)\s+(\d+)\s*/&&!$orguser) {
	    $tot=$1;
	    $cur=$2;
	    $setlen->execute($1);
	    $setpos->execute($2);
	}
    }
    close PLAYER;

    if ($tot/2>$cur&&$deltop) {
	$updtop->execute((0.5-$cur/$tot)*&toplist,$file);
    }
}

my $quit=0;

sub realquit {
    print "Preparing to quit\n";
    $quit=1;
}

$SIG{ALRM} = \&realquit;
$SIG{PIPE} = 'IGNORE';

if ($orguser) {
    my $queue=fetchsetting("PRIVATEQUEUE",$orguser);
    if (!$queue) {
	print STDERR "User doesn't have a private queue\n";
	exit 2;
    }
    $user=" $queue";
} else {
    my @channels=split(/[^A-Za-z0-9]+/,fetchsetting("AUTOSTART"));
    for my $channel (@channels) {
	if (length($channel)) {
	    print "Started $channel\n";
	    my $pid=fork;
	    if ($pid==0) {
		exec $0,$channel;
		die "Couldn't start $channel";
	    }
	    push(@pids,$pid);
	}
    }
}

$0="Unnamed play queue";

for my $pid (split(/\s+/,`$PIDOF "playit.pl$user"`)) {
    if ($pid != $$) {
	print STDERR "Player already running\n";
	exit 2;
    }
}

$0="playit.pl$user";

my ($iceip,$iceport,$icepass,$icequeue);
$iceip=fetchsetting("ICEIP");
$iceport=fetchsetting("ICEPORT");
$icepass=fetchsetting("ICEPASS");
$icequeue=fetchsetting("PRIVATEQUEUE");

if ($iceip&&$iceport) {
    $SIG{CONT}=\&resignal;
    $SIG{INT}=\&resignal;
    $SIG{HUP}=\&resignal;
    $SIG{TSTP}=\&stopsig;
    $SIG{TERM}=\&resignal;
    $ENV{"JUKEBOXBIN"}=$JUKEBOXBIN;
    while(!$quit) {
	if (!($pid=fork)) {
	    exec ("$JUKEBOXBIN/globecomice",$iceip,$iceport,$icepass,$orguser,$icequeue);
	    die "Failed to run globecomice\n";
	}
	while(wait>=0) {
	}
	sleep(5);
    }
} elsif (!$user) { # Can only play global queue on global user
    my $plasel=$dbh->prepare("select program,flags from formats where id = ?");

    while(!$quit) {
	open (OUT,"$JUKEBOXBIN/getnextmp3.pl $orguser|") || die "Coultn'd get next";
	my $file=<OUT>;
	if ($file) {
	    chomp $file;
	    my $format=<OUT>;
	    chomp $format;
	    close OUT;
	    $plasel->execute($format);
	    my ($prog,$flags)=$plasel->fetchrow_array;
	    if ($prog&&length($file)>0) {
		my $current;
		my $currentlink;
		if ($current=fetchsetting("CURRENT")) {
		    if (-l "$SITE/$current") {
			unlink("$SITE/$current");
		    }
		    symlink($file,"$SITE/$current");
		    $currentlink="$SITE/$current";
		}
		if ($prog=~/^\//) {
		    startplayer("$prog",$file,$flags);
		} else {
		    startplayer("$JUKEBOXBIN/$prog",$file,$flags);
		}
		if ($currentlink&& -l $currentlink) {
		    unlink($currentlink);
		    $currentlink="";
		}
	    } elsif (!$prog) {
		print STDERR "Couldn't find program for $format\m";
	    }
	} else {
	    sleep(5);
	}
    }
    system("$JUKEBOXBIN/getnextmp3.pl",$user,"quit");

    for my $pid (@pids) {
	print "Sending quit signal to $pid\n";
	kill "ALRM",$pid;
    }

    sleep (1);

    for my $pid (@pids) {
	printf "Sending termination signal to $pid\n";
	kill "TERM",$pid;
    }
}
