# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use strict;
use defsjukebox; # Get the settings entered by the user during configuration

$0="playmp3.pl";

if (@ARGV!=1) {
    exit 1;
}

use IPC::Open2;

my $pid=open2(*PLAYIN,*PLAYOUT,$MPG123,"-R","dummy");

print PLAYOUT "LOAD ".$ARGV[0]."\n";

my $fp10s;

my $playing=1;
my @output;

$|=1;

while(<PLAYIN>) {
    if (/^\@F\s+(\d+)\s+(\d+)\s+(\d*\.?\d+)\s+(\d*\.?\d+)\s*$/) {
        if ($4>0) {
	    $fp10s=int(10*$2/$4);
	    last;
        } elsif ($3>0) {
            $fp10s=int(10*$1/$3);
	    last;
        }
    }
    if (/^\@P 0/) {
        print PLAYOUT "QUIT\n";
    }
}

sub play {
    if (!$playing) {
	$playing=1;
	print PLAYOUT "PAUSE\n";
    }
}
sub pause {
    if ($playing) {
	$playing=0;
	push(@output,"PAUSE");
    }
}
sub forward {
    push(@output,"JUMP +$fp10s");
}
sub backward {
    push(@output,"JUMP -$fp10s");
}
sub quit {
    print PLAYOUT "QUIT\n";
}

#$SIG{CHLD}=\&quit;
$SIG{TSTP}=\&pause;
$SIG{CONT}=\&play;
$SIG{INT}=\&forward;
$SIG{HUP}=\&backward;
$SIG{TERM}=\&quit;
$SIG{CHLD}=\&quit;

my $len;
my $pos;
my $llen=0;
my $lpos=0;

while(<PLAYIN>) {
    if (@output) {
	print PLAYOUT shift(@output)."\n";
    }
    if (/^\@F \d+ \d+ (\S+) (\S+)$/) {
	$len=int($1+$2);
	$pos=int($1);
	if ($llen!=$len||$lpos!=$pos) {
	    print "$len $pos\n";
	    $llen=$len;
	    $lpos=$pos;
	}
    }
    if (/^\@P 0/) {
	print PLAYOUT "QUIT\n";
    }
}

close PLAYIN;
