/* ====================================================================
 * Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by GlobeCom GCAB AB
 *    <http://www.globecom.net/> for use in GlobeCom Jukebox."
 *
 * 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    hpj@globecom.net.
 *
 * 5. Products derived from this software may not be called "GlobeCom
 *    Jukebox" nor may "GlobeCom" appear in their names without prior written
 *    permission of the GlobeCom GCAB AB.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by GlobeCom GCAB AB
 *    <http://www.globecom.net/> for use in GlobeCom Jukebox."
 *
 * 7. The software cannot be redistributed or used for commercial purposes
 *    without a license from MpegTV, LLC <http://www.xaudio.com/>
 *
 * THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */

#include <cdaudio.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <syslog.h>

#define DEBUG_LOG 0

#define Facility  LOG_DAEMON
#define Level     LOG_INFO

int Priority = LOG_MAKEPRI(LOG_DAEMON, LOG_INFO);
extern int errno;

int my_print_cddb_matches(int cd_desc, struct disc_data *data, int matchindex);
int my_cddb_store(char *email);

void usage(void)
{
  exit(1);
}

int device;
int track;
struct disc_status status;

void Pause(int a)
{
  cd_pause(device);
}

void Cont(int a)
{
  cd_resume(device);
}

void Quit(int a)
{
  cd_stop(device);
  exit(0);
}

void Forward(int a)
{
  struct disc_timeval time;

  time.minutes = 0;
  time.seconds = 10;
  time.frames = 0;
  
  cd_track_advance(device, track, time);

  cd_poll(device,&status);
}

void Rewind(int a)
{
  struct disc_timeval time;

  time.minutes = 0;
  time.seconds = -10;
  time.frames = 0;
  
  cd_track_advance(device, track, time);

  cd_poll(device,&status);
}

int main(int argc,char **argv)
{
  int cddb;
  int cddbmatches, cddbstore;

#if DEBUG_LOG
openlog("playcd",LOG_NDELAY,Facility);
#endif
 
  if(argc < 2) {
    usage();
  }

 
  track = atoi(argv[1]);
  cddb = !strcmp(argv[1],"cddb");
  cddbmatches = !strcmp(argv[1],"cddbmatches");
  cddbstore = !strcmp(argv[1], "cddbsubmit");

  device=cd_init_device(getenv("CDROM")?getenv("CDROM"):"/dev/cdrom");
  if (device<0)
    perror("Couldn't open CD\n");
  else if (track) {
    cd_play_track(device,track,track);
    signal(SIGINT,Forward);
    signal(SIGQUIT,Quit);
    signal(SIGTERM,Quit);
    signal(SIGHUP,Rewind);
    signal(SIGABRT,Quit);
    signal(SIGSTOP,Pause);
    signal(SIGTSTP,Pause);
    signal(SIGCONT,Cont);
    for (;;) {
      sleep(1);
      cd_poll(device,&status);
      if (status.status_mode!=CDAUDIO_PLAYING&&status.status_mode!=CDAUDIO_PAUSED) {
	exit(0);
      }
    }
  } else if (cddb || cddbmatches) {
    struct disc_data data;
    struct cddb_entry entry;
    struct disc_info info;
    char buf[10];
    int i, result;

    cd_stat(device,&info); 
    if (info.disc_total_tracks) {
      memset(&data, 0, sizeof(data));

      result = my_print_cddb_matches(device, &data, (argc == 3) ? atoi(argv[2]) : 1);  // my version of cddb_read_data. If in index is 0 it gets
                                                             //      the list of matches and prints them INSIDE THE FUNCTION. A bit of a hack
      data.data_id = cddb_discid(device);// generate the 0x12345678 format disc ID.

      if(result == -1)   // we didn't get any matches, so nothing was printed in my_print_cddb_matches ( ie because CDDB lookup is disabled )
	printf("0\n");   // output the number of matches so the PHP stuff knows what to do.

      printf("%s\n",data.data_title); // JBM. Print the CDDB disc ID as hex, the cd index id and genre ( category ) as text.
      printf("%x\n%s\n%s\n%d\n",data.data_id,data.data_cdindex_id, cddb_genre(data.data_genre), data.data_revision);
      for (i=0;i<info.disc_total_tracks;i++) {
	char *artist=data.data_track[i].track_artist;
	if (!*artist)
	  artist=data.data_artist;
	if (!*artist) {
	  artist=buf;
	  sprintf(artist,"%d",i+1);
	}
	printf("%s\n",artist);
	printf("%s\n",data.data_track[i].track_name);
        printf("%d\n", info.disc_track[i].track_length.minutes * 60 +
                       info.disc_track[i].track_length.seconds);
      }
    }
  } else if (cddbstore) {
    if(argc > 2)
      my_cddb_store(argv[2]);

  } else
    usage();
}



//cddb_read_data(int cd_desc, struct disc_data *data)

// if matchindex == 0
// print the list of matches from the lookup. Based on cddb_read_data in libcdaudio/cddb.c
// otherwise return the match indexed by matchindex in a structure just like cddb_read_data

// coped from cddb_read_data in ccdb.c as part of libcdaudio

int my_print_cddb_matches(int cd_desc, struct disc_data *data, int matchindex)
{
   int sock = -1, index, i;
   char *http_string;
   struct disc_info disc;
   struct cddb_entry entry;
   struct cddb_hello hello;
   struct cddb_query query;
   struct cddb_conf conf;
   struct cddb_server *proxy_ptr;
   struct cddb_serverlist list;
   
   if(cd_stat(cd_desc, &disc) < 0)
     return -1;
   
   if(!disc.disc_present)
     return -1;

   if((http_string = malloc(512)) == NULL)
     return -1;
   
   if((proxy_ptr = (struct cddb_server *)malloc(sizeof(struct cddb_server))) == NULL) {
      free(http_string);
      return -1;
   }
   
   cddb_read_serverlist(&conf, &list, proxy_ptr);
   if(!conf.conf_access) {
      free(http_string);
      free(proxy_ptr);	  
      return -1;
   }
   if(!conf.conf_proxy) {
      free(proxy_ptr);
      proxy_ptr = NULL;
   }
   
   if(list.list_len < 1) {
      free(http_string);
      return -1;
   }
   
   strncpy(hello.hello_program, "Globecom-Jukebox", 256);
   strncpy(hello.hello_version, "4.0", 256);
   
   index = 0;
	
   /* Connect to a server */
   do {
      switch(list.list_host[index].host_protocol) { 
       case CDDB_MODE_CDDBP: 
	 sock = cddb_connect_server(list.list_host[index++], proxy_ptr, hello);
	 break;
        case CDDB_MODE_HTTP:
	 sock = cddb_connect_server(list.list_host[index++], proxy_ptr, hello, http_string, 512);
	 break;
       case CDINDEX_MODE_HTTP:
	 sock = cdindex_connect_server(list.list_host[index++], proxy_ptr, http_string, 512);
      }
   } while(index < list.list_len && sock == -1);
   
   if(sock < 0) {
      if(conf.conf_proxy)
	free(proxy_ptr);
  
      free(http_string);
      return -1;
   }
	
   index--;
	
   /* CDDB Query, not nessecary for CD Index operations */
   switch(list.list_host[index].host_protocol) {
     case CDDB_MODE_CDDBP:
       if(cddb_query(cd_desc, sock, CDDB_MODE_CDDBP, &query) < 0) {
	  if(conf.conf_proxy) free(proxy_ptr);
	  free(http_string);
          return -1;
       }
      break;
     case CDDB_MODE_HTTP:
       if(cddb_query(cd_desc, sock, CDDB_MODE_HTTP, &query, http_string) < 0) {
	  if(conf.conf_proxy) free(proxy_ptr);
	  free(http_string);
	  return -1;
       }
       shutdown(sock, 2);
       close(sock);
       
       /* We must now reconnect to execute the next command */
       if((sock = cddb_connect_server(list.list_host[index], proxy_ptr, hello, http_string, 512)) < 0) {
	  if(conf.conf_proxy) free(proxy_ptr);
	  free(http_string);
	  return -1;
       }
      break;
   }
   
   if(conf.conf_proxy) free(proxy_ptr);

   printf("%d\n", query.query_matches);

   for(i = 0; i <  query.query_matches; i++)
      printf("%s\n%s\n", query.query_list[i].list_title, query.query_list[i].list_artist);

   if(matchindex > query.query_matches)
     matchindex = query.query_matches;

   entry.entry_id = query.query_list[matchindex - 1].list_id;
   entry.entry_genre = query.query_list[matchindex - 1].list_genre;

   /* Read operation */
   switch(list.list_host[index].host_protocol) {
     case CDDB_MODE_CDDBP:
       if(cddb_read(cd_desc, sock, CDDB_MODE_CDDBP, entry, data) < 0) {
	  free(http_string);
          return -2;
       }
      
       cddb_quit(sock);
       break;
     case CDDB_MODE_HTTP:
       if(cddb_read(cd_desc, sock, CDDB_MODE_HTTP, entry, data, http_string) < 0) {
	  free(http_string);
	  return -2;
       }
      
       shutdown(sock, 2);
       close(sock);
       break;
    case CDINDEX_MODE_HTTP:
      if(cdindex_read(cd_desc, sock, data, http_string) < 0) {
	 free(http_string);
	 return -2;
      }
      
      shutdown(sock, 2);
      close(sock);
   }
   
   free(http_string);
   return 0;
}


char *getsn(char *string, int n)
{
    int       ch, i;
    char      *copy = string;
    int       count = 0;

    while ((ch = getc(stdin)) != '\n' && count < (n + 1)) {
      if(ferror(stdin)) {
#if DEBUG_LOG
	syslog(Priority, "getsn error : %s ", strerror(errno));
#endif
      }
      if(ch != EOF) {
	*copy++ = (char) ch;
	count++;
      }
      else
	clearerr(stdin);
    }

    *copy++ = 0;


#if DEBUG_LOG
    syslog(Priority, "string : %s ", string);
#endif

    return string;
}


// Take our track / artist data from the ripping form and save it into the local cddb store. This step is required before we submit
// the data to CDDB. reads a huge list from stdin, as follows :

// Track count
// Album name
// Genre
// revision
// Primary artist
// Single artist (0) or Multiple artist (1)
// optional list of artists ( track count of them, but only if multiple artist above is set, ie it's a 'various' compilation )
// list of tracks ( track count )

int my_cddb_store(char *email)
{
    struct disc_data data;
    struct disc_info info;
    int numtracks, i;
    char buffer[256], root_dir[256], genre_dir[256], file[256], exec[512];

    memset(&data, 0, sizeof(data));

    cd_stat(device,&info); 
    data.data_id = cddb_discid(device);// generate the 0x12345678 format disc ID.

#if DEBUG_LOG
    syslog(Priority, "cd = %d\n", data.data_id);
#endif
    setvbuf(stdin, 0, _IONBF, 0);

    getsn(buffer, 256);
    numtracks = atoi(buffer);

    if (info.disc_total_tracks == numtracks) {

      getsn(buffer, 256);
      strncpy(data.data_title, buffer, 256);/* Disc title */

      getsn(buffer, 256);
      data.data_genre = cddb_genre_value(buffer);	       /* Disc genre */

      getsn(buffer, 256);
      data.data_revision = atoi(buffer);      /* CDDB revision (incremented with each submit) */

      getsn(buffer, 256);
      strncpy(data.data_artist, buffer, 256);/* Artist title */

      getsn(buffer, 256);
      data.data_artist_type = atoi(buffer);	       /* Single or multiple artist CD */
      
      if(data.data_artist_type) {
	for(i = 0; i <  numtracks; i++) {
	  getsn(buffer, 256);
	  strncpy(data.data_track[i].track_artist, buffer, 256);
	}
      }

      for(i = 0; i <  numtracks; i++) {
	getsn(buffer, 256);
        strncpy(data.data_track[i].track_name, buffer, 256);
      }

      cddb_write_disc_data(device, data);
      
      // now read the file just written.

      snprintf(root_dir, 256, "%s/.cddb", getenv("HOME"));
      snprintf(genre_dir, 256, "%s/%s", root_dir, cddb_genre(data.data_genre));
      snprintf(file, 256, "%s/%08lx",  genre_dir, data.data_id);
#if DEBUG_LOG
    syslog(Priority, "email : %s", email);
#endif
    
      snprintf(exec, 512, "cat %s | mail -s \"cddb %s %08lx\" %s", file, cddb_genre(data.data_genre), data.data_id, email);

      if((i = system(exec)) != 0) {
#if DEBUG_LOG
    syslog(Priority, "system call to mail failed : %d", i);
#endif
	return -1;
      }
    } else {
      perror("info.disc_total_tracks != numtracks\n");
      return(0);
    }

    return(0);
}
